// TO RUN IN PRODUCTION MODE
// first set up the environment variable:
// set NODE_ENV=production

// Use "npm start" to run
trace = function(str)
{
    if (false) console.log(str);
}

const { app, BrowserWindow, Menu, dialog } = require('electron')
const path = require('path');
const fs = require('fs');
const { download } = require('electron-dl');
const {shell} = require('electron');

//For config.json in app data folder to store domain name
const Store = require('electron-store');
const store = new Store();

// command-line parameters?
let url;
var foundUrl = false;
for (var c = 0; c < process.argv.length; c++)
{
	if (process.argv[c].toLowerCase() == "-url")
	{
		foundUrl = true;
	}
	else if (foundUrl)
	{
		url = process.argv[c];
		break;
	}
}
if (!foundUrl)
{
  //url = path.join(__dirname, 'index.html');
  url = 'https://itaca.edu.gva.es/';
}
trace("Starting with URL -> " + url);

let mainWindow;
var appReady = false;

app.on('window-all-closed', function () {
  trace("All windows closed");
  //if (process.platform != 'darwin')
  {
    app.quit();
  }
});

//app.commandLine.appendSwitch('ignore-certificate-errors', 'true');
//app.setAccessibilitySupportEnabled(true);

let ppapi_flash_path;
if (process.platform == 'win32') {
  ppapi_flash_path = path.join(__dirname, '.\\plugin\\HCSFP.dll');
//ppapi_flash_path = "C:\\Users\\Admin\\AppData\\Local\\Google\\Chrome\\User Data\\PepperFlash\\32.0.0.363\\pepflashplayer.dll";
} else if (process.platform == 'linux') {
  ppapi_flash_path = path.join(__dirname, './lib/libHCSFP.so');
  app.commandLine.appendSwitch('no-sandbox');
} else if (process.platform == 'darwin') {
  ppapi_flash_path = path.join(__dirname, './plugin/PepperFlashPlayer.plugin');
}

//trace("Path for FP = " + ppapi_flash_path);
app.commandLine.appendSwitch('ppapi-flash-path', ppapi_flash_path);

// common event handlers

onWillDownload = function (event, item, webContents) {
    trace('Will download file: ' + item.getURL() + ' - ' + item.getMimeType());
    // run default desktop app for pdf without download requests
    if (item.getMimeType() === 'application/pdf') {
        var PDFfilename = path.join(app.getPath("temp"), item.getFilename());
        var f = PDFfilename;
        var extension = path.extname(PDFfilename);
        trace('extension: ' + extension);
        var n = 1;
        while (fs.existsSync(f)) {
            f = PDFfilename.replace(extension, `(${n})` + extension);
            n += 1;
        }
        PDFfilename = f;
        item.setSavePath(PDFfilename);
        item.once('done', (event, state) => {
            if (state === 'completed') {
                trace('Download successfully')
                const choice = dialog.showMessageBoxSync(mainWindow, { // need 'Sync' for newer Electron versions
                    type: 'warning',
                    buttons: ['Yes', 'No'],
                    title: 'Permiso para abrir fichero pdf',
                    message: 'Fichero: ' + PDFfilename.replace(/^.*[\\\/]/, '') + ' descargado en la carpeta ' + app.getPath("temp") + '.\nDesea abrir el fichero?',
                    defaultId: 0,
                    cancelId: 1
                });
                if (choice === 1) {
                    trace(`The application requested permission to open '${PDFfilename}'. User blocked this request.`);
                    event.preventDefault();
                } else {
                    trace(`The application requested permission to open '${PDFfilename}'. User allowed this request.`);
                    try {
                        // MacOS needs a file:// based URL; Windows needs the \ changed to / for this to work.
                        var fileURL = "file://" + encodeURI(item.getSavePath().replace(/\\/g, "/"));
                        trace("Opening: " + fileURL);
                        shell.openExternal(fileURL)
                            .then(() => { trace('Shell openExternal succeeded'); })
                            .catch(err => { trace(err); });
                    }
                    catch (shellErr) {
                        trace(shellErr);
                    }
                }
            } else {
                trace(`Download failed: ${state}`)
            }
        })
    } else if (item.getMimeType() === 'text/csv' || item.getMimeType() === 'application/vnd.ms-excel' || item.getMimeType() === 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet') {
        var bWindow = BrowserWindow.fromWebContents(webContents);
        bWindow.close();

    } else {
        var bWindow = BrowserWindow.fromWebContents(webContents);

        event.preventDefault();

        var parentWindow = mainWindow;
        var options = {
            'width': 1650,
            'height': 950,
            'title': 'AVISO',
            'webPreferences': {
                'plugins': true,
                'nativeWindowOpen': true
            }
        }
        options.width = parentWindow.getSize()[0];
        options.height = parentWindow.getSize()[1];
        var newWin = new BrowserWindow(options);
        if (parentWindow.isMaximized()) newWin.maximize();
        newWin.loadURL(item.getURL());
        event.newGuest = newWin;

        if (!bWindow.webContents.getURL()) bWindow.close(); // blank window just for the download -> close it
    }
};

onNewWindow = function(event, url, frameName, disposition, options)
{
    event.preventDefault();
    if (checkForPDF(url))
    {
        trace("Ignoring new window request due to PDF download");
        return;
    }

    // create a window manually so that we can add our event handlers
    trace("Creating a new window");
    trace(options);
    var parentWindow = mainWindow;
    if (options.webPreferences && options.webPreferences.openerId && BrowserWindow.fromId(options.webPreferences.openerId))
    {
        parentWindow = BrowserWindow.fromId(options.webPreferences.openerId);
    }

    options.width = parentWindow.getSize()[0];
    options.height = parentWindow.getSize()[1];
    var newWin = new BrowserWindow(options);
    if (parentWindow.isMaximized()) newWin.maximize();
    if (!options.webContents) newWin.loadURL(url);
    event.newGuest = newWin;
    //newWin.toggleDevTools();
    newWin.webContents.on('new-window', onNewWindow);
    newWin.webContents.on('plugin-crashed', onPluginCrashed);
    newWin.webContents.on('will-prevent-unload', (event) =>
    {
        trace("Preventing unload!");
        const choice = dialog.showMessageBoxSync(newWin, { // need 'Sync' for newer Electron versions
            type: 'question',
            buttons: ['Leave this page', 'Stay on this page'],
            title: 'Are you sure you want to leave this page?',
            message: 'Your changes may be lost. Do you want to continue?',
            defaultId: 0,
            cancelId: 1
        });
        const leave = (choice === 0);
        trace("Choice -> " + choice);
        if (leave) event.preventDefault();
    });
};

onPluginCrashed = function(event, name, version)
{
    trace("Plugin crashed: " + event.sender.getURL());
    var win = BrowserWindow.fromWebContents(event.sender);
    //trace(win); For some reason we can't just load a new URL into the window.. :-(
    dialog.showMessageBoxSync(win, { // need 'Sync' for newer Electron versions
            title: 'Error in plug-in',
            message: 'Plug-in has crashed: window will be closed.'
    });
	//win.loadURL("https://www.google.com");
    win.close();
};

startElectron = function()
{
  trace("Starting electron...");
  if (!appReady)
  {
      trace("Waiting for electron app");
      return;
  }
  mainWindow = new BrowserWindow({
    'width': 1650,
    'height': 950,
    'title': 'ITACA',
    'webPreferences': {
      'plugins': true,
      'nativeWindowOpen': true
    }
  });

  try {
	//Add domain names from config.json to array
	if(store.size > 0)
	{
		var i;
		for (i = 0; i<store.size; i++) {
		ignore_certificate_errors.push(store.get(i.toString()));
		}
	}
    mainWindow.loadURL(url);
    //mainWindow.toggleDevTools();
    mainWindow.webContents.on('new-window', onNewWindow);
    mainWindow.webContents.on('plugin-crashed', onPluginCrashed);
    mainWindow.webContents.session.on('will-download', onWillDownload);
	mainWindow.webContents.on('will-prevent-unload', (event) =>
    {
        trace("Preventing unload!");
        const choice = dialog.showMessageBoxSync(mainWindow, { // need 'Sync' for newer Electron versions
            type: 'question',
            buttons: ['Leave this page', 'Stay on this page'],
            title: 'Are you sure you want to leave this page?',
            message: 'Your changes may be lost. Do you want to continue?',
            defaultId: 0,
            cancelId: 1
        });
        const leave = (choice === 0);
        trace("Choice -> " + choice);
        if (leave) event.preventDefault();
    });

	const template = [
	{
	 label: "Options",
	 submenu: [
	 {
	   label: 'Home',
	   click() { mainWindow.loadURL(url); }
	 },
	 {
      label: 'Clear cache',
	   click() {
		   mainWindow.webContents.session.clearCache().then(() => {
				aboutDialogOptions = {
					type: "info",
					title: "Message",
					buttons: ["OK"],
					message: "Clean-up completed",
				};
				dialog.showMessageBoxSync(mainWindow, aboutDialogOptions);
			});

			store.clear(); // Clear domain name from config.json
	   }
	 },
	 {
	   label: 'Zoom +',
	   click() {
			var currentZoom = mainWindow.webContents.getZoomFactor();
			mainWindow.webContents.setZoomFactor(currentZoom + 0.1);
	   }
	 },
	   {
	   label: 'Zoom -',
	   click() {
			var currentZoom = mainWindow.webContents.getZoomFactor();
			mainWindow.webContents.setZoomFactor(currentZoom - 0.1);
	   }
	 },
	 {
	    role: 'quit'
	 }
	 ]
	}];
    const menu = Menu.buildFromTemplate(template);
    Menu.setApplicationMenu(menu);
  }
  catch (e) {
    console.error(e);
  }
}

app.on('ready', ()=>{
    appReady = true;
    startElectron();
});

var ignore_certificate_errors = [];

app.on('certificate-error', (event, webContents, url, error, certificate, callback) =>
{
	trace(url);
	trace(error);
	trace(certificate);
	var okayToContinue = false;
	var domainName;
	// get just the domain name
	if (url.indexOf("https://") == 0)
	{
		domainName = url.substring(8, url.length).split("/")[0];
		if (ignore_certificate_errors.indexOf(domainName) >= 0) okayToContinue = true;
	}
	if (!okayToContinue)
	{
		var win = BrowserWindow.fromWebContents(webContents);
		const choice = dialog.showMessageBoxSync(win, { // need 'Sync' for newer Electron versions
			type: 'warning',
			buttons: ['Cancel navigation', 'Ignore error and proceed to page'],
			title: 'Certificate Error',
			message: 'This webserver has provided an invalid certificate: ' + url + '\nYour connection is not private [' + error + ']\n\n' +
				'If you are sure that this website is safe, you may ignore this security warning and continue.\n',
			defaultId: 0,
			cancelId: 1
		});
		const continueNavigation = (choice === 1);
		trace("Choice -> " + choice);
		if (continueNavigation)
		{
			okayToContinue = true;
			if (domainName)
			{
				ignore_certificate_errors.push(domainName);
				store.set(((store.size)+1).toString(), domainName.toString());
			}
		}
	}
	if (okayToContinue)
	{
		trace("Cancelling certificate error");
		event.preventDefault();
		callback(true);
	}
	else
	{
		callback(false);
	}
});

/*
if (process.env.NODE_ENV !== 'production') {
  const mainMenuTemplate = [{
    label: "Developer Tools",
    submenu: [
      {
        label: "Toggle DevTools",
        accelerator: process.platform == 'darwin' ? 'Command+I' : 'Ctrl+I',
        click(item, focusedWindow) {
          focusedWindow.toggleDevTools();
        }
      },
      {
        role: 'reload'
      }]
  }];

  const mainMenu = Menu.buildFromTemplate(mainMenuTemplate);
  Menu.setApplicationMenu(mainMenu);
}
*/

//////////////// PDF download ////////////////////////
checkForPDF = function (url) {
    var origURL = url;
    trace("Checking url for PDF: " + url);
    var idxQuery = url.indexOf('?');
    if (idxQuery > 0) url = url.substr(0, idxQuery);

    url = url.toLowerCase();
    if (url.endsWith('.pdf')) {
        download(BrowserWindow.getFocusedWindow(), origURL, {
            directory: app.getPath("temp")
        });
        // Next code located in onWillDownload callback
        return true;
    }
    return false;
}
